require 'redmine'
require 'application_helper'

module TraceabilityMatrix
   
  ##############################################################################

  Redmine::WikiFormatting::Macros.register do
    desc "Traceability matrix between two list of issues, as a simple cross reference table.\n\n" +
         "+Syntax:+\n" +
         "<pre>{{traceability_matrix_short(issue_query_id_row,issue_query_id_col,[options])}}</pre>\n\n" +
         "+Parameters+\n" +
         "* _issue_query_id_row_ : id of the issue query to get the list of issues to rows \n" +
         "* _issue_query_id_col_ : id of the issue query to get the list of issues to cols \n\n" +
         "+Options:+\n" +
         "* -w = number : Specifies the width of the tables, in number of columns. The traceabily matrix may be spli into several tables.\n" +
         "* -s : Displays the status of issues\n\n" +
         "+Examples:+\n" +
         "<pre>{{traceability_matrix_short(1,2)}} ->  Show the matrix using issue queries with id 1 and 2\n" +
         "{{traceability_matrix_short(20,144,-w=20)}} Show the matrix, split into tables that contains 20 columns\n" +
         "{{traceability_matrix_short(20,144,-s)}} Each issue, in rows and columns displays its status\n</pre>\n"
         
               
    macro :traceability_matrix_short do |obj, args|
    
      ## Gestion des arguments ##      
      if args.empty?
        # Set default value
        query_row_id = Setting.plugin_traceability_matrix['tracker0']
        query_col_id = Setting.plugin_traceability_matrix['tracker1']
      else
        if Query.find_by_id(args[0].strip) == nil
          return 'Error : Unknown query id for rows'
        else
          query_col_id = Query.find(args[0].strip)     
        end
        if Query.find_by_id(args[1].strip) == nil
          return 'Error : Unknown query id for columns'
        else
          query_row_id = Query.find(args[1].strip) 
        end
      end
      
      # default values
      split_nb_cols = 20
      show_status = false
      
      args[2..-1].each do |arg|
        case arg
        when /^-w/
          # width of array, in number of columns
          split_nb_cols = arg.delete('-w=').strip
          puts "nb columns param = " + split_nb_cols
        when /^-s/
          # show status of issues
          show_status = true
        else
          puts "sinon" + arg
        end
      end

      ## Execution du controlleur ##
      mt_ctrl = MtController.new  
      
      mt_ctrl.init_macro_context(@project)
      mt_ctrl.get_trackers(query_row_id, query_col_id)  
      mt_ctrl.build_list_of_issues
      
      disp = String.new
      disp << render(:partial => 'mt/mt_synthese', :locals => {:issue_cols => mt_ctrl.issue_cols, :query_cols => mt_ctrl.query_cols,
                     :issue_rows => mt_ctrl.issue_rows, :query_rows => mt_ctrl.query_rows, :issue_pairs => mt_ctrl.issue_pairs,
                     :not_seen_issue_cols => mt_ctrl.not_seen_issue_cols, :split_nb_cols => split_nb_cols.to_i, :show_status => show_status})
      
      return disp.html_safe

    end # Fin macro
    
    
    desc "Detailed traceability matrix between two list of issues.\n\n" +
         "+Syntax:+\n" +
         "<pre>{{traceability_matrix_short(issue_query_id_row,issue_query_id_col,[options])}}</pre>\n\n" +
         "+Parameters+\n" +
         "* _issue_query_id_row_ : id of the issue query to get the list of issues to rows \n" +
         "* _issue_query_id_col_ : id of the issue query to get the list of issues to cols \n\n" +
         "+Options:+\n" +
         "* -d : Displays the description of issues.\n\n" +
         "+Examples:+\n" +
         "<pre>{{traceability_matrix_detailed(1,2)}} ->  Show the matrix using issue queries with id 1 and 2\n" +
         "{{traceability_matrix_detailed(20,144,-d)}} Show the matrix, with description of issues\n</pre>\n"
         
    macro :traceability_matrix_detailed do |obj, args|
    
      ## Gestion des arguments ##      
      if args.empty?
        # Set default value
        query_row_id = Setting.plugin_traceability_matrix['tracker0']
        query_col_id = Setting.plugin_traceability_matrix['tracker1']
      else
        if Query.find_by_id(args[0].strip) == nil
          return 'Error : Unknown query id for rows'
        else
          query_col_id = Query.find(args[0].strip)     
        end
        if Query.find_by_id(args[1].strip) == nil
          return 'Error : Unknown query id for columns'
        else
          query_row_id = Query.find(args[1].strip) 
        end
      end
      
      option_display_description = false
      
      args[2..-1].each do |arg|
        case arg
        when /^-d/
          # display description of issues
          option_display_description = true
        else
          puts "sinon" + arg
        end
      end

      ## Execution du controlleur ##
      mt_ctrl = MtController.new  
      
      mt_ctrl.init_macro_context(@project)
      mt_ctrl.get_trackers(query_row_id, query_col_id)  
      mt_ctrl.build_list_of_issues
      
      disp = String.new
      disp << render(:partial => 'mt/mt_details', :locals => {:issue_cols => mt_ctrl.issue_cols, :query_cols => mt_ctrl.query_cols,
                     :issue_rows => mt_ctrl.issue_rows, :query_rows => mt_ctrl.query_rows, :issue_pairs => mt_ctrl.issue_pairs,
                     :not_seen_issue_cols => mt_ctrl.not_seen_issue_cols, :option_display_description => option_display_description})
      return disp.html_safe

    end # Fin macro
        
  end

end

################################################################################
